package pipeline.vertex;

import java.util.List;

import javax.vecmath.Color3f;
import javax.vecmath.Vector2f;
import javax.vecmath.Vector3f;
import javax.vecmath.Vector4f;

import pipeline.Pipeline;
import pipeline.PointLight;
import pipeline.math.Matrix4f;
import pipeline.math.Util;
import pipeline.misc.Vertex;

/**
 * This is the same triangle processor as FragmentShadedTP, but it also
 * interpolates texture coordinates, allowing for texturing to be done in the
 * fragment stage.
 * 
 * @author ags
 */
public class TexturedPhongVP extends VertexProcessor {
	protected Matrix4f mvp = new Matrix4f(); // Model View Projection Matrix
    protected Matrix4f mv = new Matrix4f(); // Model View Matrix

    protected List<PointLight> lights;
    protected float ambientIntensity;
    protected Color3f specularColor;
    protected float specularExponent;
    
    /**
     * @see VertexProcessor#compatibilityIndex()
     */
    public int compatibilityIndex() {
        return 11;
    }
    
    /**
     * @see VertexProcessor#updateTransforms(Pipeline)
     */
    public void updateTransforms(Pipeline pipe) {
        mvp.set(pipe.modelviewMatrix);
        mvp.leftCompose(pipe.projectionMatrix);
        mvp.leftCompose(pipe.viewportMatrix);

        mv.set(pipe.modelviewMatrix);
    }

    public void vertex(Vector3f v, Color3f c, Vector3f n, Vector2f t,
            Vertex output) {
    	// TODO
    }

    /**
     * @see VertexProcessor#triangle(Vector3f[], Color3f[], Vector3f[],
     *      Vector2f[], Vertex[])
     */
    public void triangle(Vector3f[] vs, Color3f[] cs_ign, Vector3f[] ns,
            Vector2f[] ts, Vertex[] outputs) {

        for (int k = 0; k < 3; k++) {
            vertex(vs[k], cs_ign[k], ns[k], ts[k], outputs[k]);
        }
    }

}
